"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = require("ramda");
/**
 * @param {import('extract-pg-schema').TableOrView[]} models
 * @param {string[]} userTypes
 * @param {import('./generateModelFile').Nominators} nominators
 * @returns {string[]}
 */
function generateIndexFile(models, userTypes, nominators) {
    // const tc = recase(casings.sourceCasing, casings.typeCasing);
    // const pc = recase(casings.sourceCasing, casings.propertyCasing);
    // const fc = recase(casings.sourceCasing, casings.filenameCasing);
    var isFixed = function (m) { return m.type !== 'table' || m.tags['fixed']; };
    var hasIdentifier = function (m) {
        return ramda_1.filter(function (c) { return c.isPrimary; }, m.columns).length === 1;
    };
    var creatableModels = ramda_1.reject(isFixed, models);
    var modelsWithIdColumn = ramda_1.filter(hasIdentifier, models);
    var importLine = function (m) {
        var importInitializer = !isFixed(m);
        var importId = hasIdentifier(m);
        var additionalImports = importInitializer || importId;
        var givenName = nominators.modelNominator(m.name);
        var fileName = nominators.fileNominator(givenName, m.name);
        if (!additionalImports) {
            return "import " + givenName + " from './" + fileName + "';";
        }
        else {
            var imports = __spreadArray(__spreadArray([], (importInitializer
                ? [nominators.initializerNominator(givenName, m.name)]
                : [])), (importId ? [nominators.idNominator(givenName, m.name)] : []));
            return "import " + givenName + ", { " + imports.join(', ') + " } from './" + fileName + "';";
        }
    };
    var exportLine = function (m) {
        var givenName = nominators.modelNominator(m.name);
        var exportInitializer = !isFixed(m);
        var exportId = hasIdentifier(m);
        var exports = __spreadArray(__spreadArray([
            givenName
        ], (exportInitializer
            ? [nominators.initializerNominator(givenName, m.name)]
            : [])), (exportId ? [nominators.idNominator(givenName, m.name)] : []));
        return "  " + exports.join(', ') + ",";
    };
    var lines = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], ramda_1.map(importLine, models)), ramda_1.map(function (t) {
        return "import " + nominators.typeNominator(t) + " from './" + nominators.fileNominator(nominators.typeNominator(t), t) + "';";
    }, userTypes)), [
        '',
        'type Model ='
    ]), ramda_1.map(function (model) { return "  | " + nominators.modelNominator(model.name); }, models)), [
        '',
        'interface ModelTypeMap {'
    ]), ramda_1.map(function (model) { return "  '" + model.name + "': " + nominators.modelNominator(model.name) + ";"; }, models)), [
        '}',
        '',
        'type ModelId ='
    ]), ramda_1.map(function (model) {
        return "  | " + nominators.idNominator(nominators.modelNominator(model.name), model.name);
    }, modelsWithIdColumn)), [
        '',
        'interface ModelIdTypeMap {'
    ]), ramda_1.map(function (model) {
        return "  '" + model.name + "': " + nominators.idNominator(nominators.modelNominator(model.name), model.name) + ";";
    }, modelsWithIdColumn)), [
        '}',
        '',
        'type Initializer ='
    ]), ramda_1.map(function (model) {
        return "  | " + nominators.initializerNominator(nominators.modelNominator(model.name), model.name);
    }, creatableModels)), [
        '',
        'interface InitializerTypeMap {'
    ]), ramda_1.map(function (model) {
        return "  '" + model.name + "': " + nominators.initializerNominator(nominators.modelNominator(model.name), model.name) + ";";
    }, creatableModels)), [
        '}',
        '',
        'export type {'
    ]), ramda_1.map(exportLine, models)), ramda_1.map(function (t) { return "  " + nominators.typeNominator(t) + ","; }, userTypes)), [
        '',
        '  Model,',
        '  ModelTypeMap,',
        '  ModelId,',
        '  ModelIdTypeMap,',
        '  Initializer,',
        '  InitializerTypeMap',
        '};',
    ]);
    return lines;
}
exports.default = generateIndexFile;
