"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.error = exports.warn = exports.quiet = exports.log = void 0;
/* eslint-disable no-console */
var LEVELS = {
    silent: 0,
    error: 1,
    warn: 2,
    quiet: 3,
    log: 4,
};
var options = {
    level: LEVELS.log,
};
function createLogger(level, method, exact) {
    if (exact === void 0) { exact = false; }
    return function (first) {
        var rest = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            rest[_i - 1] = arguments[_i];
        }
        if (level > options.level || (exact && options.level !== level))
            return;
        if (first.raw) {
            console[method](String.raw.apply(String, __spreadArray([first], rest)));
        }
        else {
            console[method].apply(console, __spreadArray([first], rest));
        }
    };
}
exports.log = createLogger(LEVELS.log, 'log', true);
exports.quiet = createLogger(LEVELS.quiet, 'log', true);
exports.warn = createLogger(LEVELS.warn, 'warn');
exports.error = createLogger(LEVELS.error, 'error');
/**
 * @param {{ level: keyof LEVELS | 0 | 1 | 2 | 3 }} config
 *  */
function logger(config) {
    switch (typeof config.level) {
        case 'string': {
            options.level = LEVELS[config.level.toLowerCase()];
            break;
        }
        case 'number': {
            options.level = config.level;
            break;
        }
        case 'boolean': {
            options.level = config.level ? LEVELS.log : LEVELS.silent;
            break;
        }
    }
}
exports.logger = logger;
logger.log = exports.log;
logger.warn = exports.warn;
logger.error = exports.error;
logger.quiet = exports.quiet;
